#ifndef BGSTATS_HH
#define BGSTATS_HH

#include <TH1D.h>
#include <TH2D.h>
#include <tools/EventStats.hh>
#include <tools/Region.hh>
#include <muon/ArrayMuonParameters.hh>
#include <sash/MonitorBase.hh>
#include <sash/Telescope.hh>
#include <sash/PointerSet.hh>

namespace Sash {
  class HESSArray;
}

namespace Background {

  typedef std::map<int,double> TelMap_t;

  /**
   * \brief Background statistics (per-run or cumulative).
   *
   * These objects are
   * generated by the BgMaker class for each run and may be added
   * together (see Background::Accumulate()) to produce the combined
   * statistics for a set of runs.
   *
   * \todo in constructor should by default set skipped flag, and only
   * set it to non-skipped in the EndRun function of the BgMaker, so
   * that if an exception is thrown the run is safely skipped.
   */
  class BgStats : public Sash::MonitorBase 
  {

  public:
    BgStats(Sash::HESSArray&
#if defined(__CINT__) || defined(CINTOBJECT)
	    = hessdummy
#endif
	    ,const char * name="");

    BgStats(Sash::HESSArray&, const BgStats &stats, const char * name);

    
    BgStats(const char *handleFunc,
	    Sash::HESSArray&
#if defined(__CINT__) || defined(CINTOBJECT)
	    = hessdummy
#endif
	    ,const char * name="");

    ~BgStats();
    
    virtual void Clear( Option_t* = "E" );
    virtual void print(std::ostream&)  const;

    GETMEMBER( RateStats, Tools::RateStats );
    GETMEMBER( OnRegion, Tools::Compound );
    GETMEMBER( OffRegion, Tools::Compound );
    GETMEMBER( OnExclusionRegion, Tools::Compound );
    GETMEMBER( OffExclusionRegion, Tools::Compound );
    GETMEMBER( IdNumber, int );
    GETMEMBER( NumberOfTelescopes, int );
    GETMEMBER( TelescopePattern, int );
    GETMEMBER( SkippedFlag, bool );
    GETMEMBER( OnFlag, bool );
    GETMEMBER( OffFlag, bool );
    GETMEMBER( NumOn, Int_t);
    GETMEMBER( NumOff, Int_t);
    GETMEMBER( NumSkippedOn, int );  
    GETMEMBER( NumSkippedOff, int ); 
    GETMEMBER( BadFlag, bool );
    GETMEMBER( NumBadOn, int );  
    GETMEMBER( NumBadOff, int );    
    
    GETMEMBER( ZenithDistOn, TH1D);
    GETMEMBER( ZenithDistOff,TH1D);
    GETMEMBER( AzimuthDistOn, TH1D);
    GETMEMBER( AzimuthDistOff,TH1D);
    GETMEMBER( OffsetDistOn, TH1D);
    GETMEMBER( OffsetDistOff,TH1D);

    GETMEMBER( ZenithOffsetDistOn, TH2D );
    GETMEMBER( ZenithOffsetDistOff,TH2D );

    GETMEMBER( IsCumulative, bool );
    GETMEMBER( ObservationPosition, Stash::Coordinate );
    GETMEMBER( MuonCorrections, Background::TelMap_t );
    GETMEMBER( MuonEntry, int );
    GETMEMBER( ArrayMuonParameters, Muon::ArrayMuonParameters );

    GETMEMBER( ConfigName, std::string );

    int GetNumTotal() const { return GetNumOn()+GetNumOff();}
    int GetNumValid() const { return GetNumTotal()-GetNumSkipped()-GetNumBad();}
    int GetNumSkipped() const { return GetNumSkippedOn()+GetNumSkippedOff();}
    int GetNumBad() const { return GetNumBadOn()+GetNumBadOff(); }

    void   AddStatsFrom(const BgStats &stats, bool online=false);
    void   ReplaceStatsFrom(const BgStats &stats, bool online=false);
    void   FillDistributions(const Stash::Coordinate &obspos,
			     const Sash::Time starttime, 
			     const Sash::Time endtime,
			     double psicut=-1,
			     bool fillon=true, 
			     bool filloff=true);
    Stash::Lambda GetMaximumOffset();
    double GetFractionOfTimeAboveOffset( double maxoffs_deg );

    double GetMeanMuonCorrection ();

    void Draw(const Option_t* opt = "");

    static double MeanAzimuth(const TH1& azimuthDist);

    double GetMeanZenithOn() const { return GetZenithDistOn().GetMean(); }
    double GetMeanZenithOff() const { return GetZenithDistOff().GetMean(); }
    double GetMeanOffsetOn() const { return GetOffsetDistOn().GetMean(); }
    double GetMeanOffsetOff() const { return GetOffsetDistOff().GetMean(); }
    double GetMeanAzimuthOn() const { return MeanAzimuth(GetAzimuthDistOn()); }
    double GetMeanAzimuthOff() const { return MeanAzimuth(GetAzimuthDistOff()); }

  private:

    void SetHistNames(const char* name);
    
    Tools::RateStats fRateStats;  //< Rate Statistics

    Tools::Compound fOnRegion; 	       //< On region, possibly compound
    Tools::Compound fOffRegion;        //< Off region, possibly compound
    Tools::Compound fOnExclusionRegion;//< Exclusion region, possibly compound
    Tools::Compound fOffExclusionRegion;//< Exclusion region, possibly compound
                                    
    Int_t            fIdNumber;	  //< Run number (or other identifier)
    int              fNumberOfTelescopes; //< number of telescopes in run (-1 if mixture)
    int              fTelescopePattern;   //< bit pattern of telescopes in run
                                     
    bool           fSkippedFlag;  //< Marker for skipped runs/ranges
    bool           fBadFlag;      //< Marker for runs which had an exception
    bool           fOnFlag;       //< Flag for ON-run
    bool           fOffFlag;      //< Flag for OFF-run
                                     
    Int_t          fNumOn;           
    Int_t          fNumOff;          
    int            fNumSkippedOn;    
    int            fNumSkippedOff;   
    int            fNumBadOn;    
    int            fNumBadOff;   

    TH1D  fZenithDistOn;  //< Zenith angle distribution for the On region
    TH1D  fZenithDistOff; //< Zenith angle distribution for the Off regions 
    TH1D  fAzimuthDistOn;  //< Azimuth angle distribution for the On region
    TH1D  fAzimuthDistOff; //< Azimuth angle distribution for the Off regions 
    TH1D  fOffsetDistOn;  //< Offset angle distribution for the On region
    TH1D  fOffsetDistOff; //< Offset angle distribution for the Off regions 

    TH2D  fZenithOffsetDistOn;  //< combined zenith and offset distribution for the On region
    TH2D  fZenithOffsetDistOff; //< combined zenith and offset distribution for the Off region

    bool fIsCumulative; //<

    Stash::Coordinate fObservationPosition; //< Observation position of run
    TelMap_t fMuonCorrections; //< Muon efficiencies used per tel
    int      fMuonEntry; //< Run number from Monitor_Run_Muon database table, corresponding to optical efficiencies in MC
    Muon::ArrayMuonParameters fArrayMuonParameters; //< Array-level muon info required for exposure calculation

    std::string fConfigName; //<stores the config name used to generate the stats

    ClassDef(Background::BgStats,9)
      // changes Version 3 to 4:
      // add fZenithOffsetDistOff
      // change version 6 to 7:
      // add number of telescopes in run
      // change version 7 to 8:
      // add bit pattern of telescopes and muon entry in run
      // change version 8 to 9:
      // added another constructor for online analysis
      };
}

#endif
